REM **************************************
REM * PICAXE Feld Hell CW Beacon Program * 
REM * Version 1.0, October 2012          *
REM **************************************

REM Various modifications & translation from Arduino C by Robert, VA3ROM, http://my.tbaytel.net/va3rom

REM Based on orginal Arduino Feld Hell C program written by:Mark VandeWettering, K6HX
REM Inspired by PICAXE CW & S/MT program code written by Steve Page, VK6HV

REM ******************************************************************
REM * Change Buffer_Length & EEPROM values for your Feld Hell beacon *
REM ****************************************************************** 
REM Store Feld Hell CW beacon information (maximum 256 characters)
REM Feld Hell, by convention, starts/ends all transmissions with 3 periods
REM We can't use numbers as only 0 to 255 allowed & Feld Hell has character values up to 322! 
eeprom 0, ("... HELLO WORLD! ...")

symbol Beacon_Length = 19				'number of characters in Feld Hell beacon - 1


REM Define rest of program variables (don't change these except for PTT port variable!)
REM Because PICAXE MCUs don't support arrays (yet) we have to simulate using sequential lookup table of 256 bytes (2 need)
symbol Hell_Character = w0				'Hell column character (7 columns)

symbol Character_Index = w1				'index to Hell character set
symbol Character_Offset = w2				'offset to Hell character column

symbol kLoop = w3						'for parsing Hell character lookup tables
symbol jChar = w4						'for holding next eeprom character to transmit

symbol jLoop = b14					'for looping through eeprom beacon characters
symbol Hell_Column_Pixel = b15			'Hell column pixel to check to switch between transmit & receive

REM Change PTT port (as required) this is for Schools Kit
symbol PTT = C.0						'port 0 (red LED using as transmitter PTT & will flash in sync with transmission)

setfreq M32							'run PICAXE 08M2 chip at 32MHz clock speed for Feld Hell 8.125ms keying

REM Start of Feld Hell CW beacon transmission sequence
Beacon:							
	
	wait 40						'5 second delay between beacons running M2 chip @ 32MHz (40/8)
	  
	REM Read EEPROM characters & determine character offset to use from lookup tables
	for jLoop = 0 to Beacon_Length		
				
		read jLoop, jChar				'get next eeprom character to transmit
	
		Character_Index = 9999			'reset character index to bogus value each time
		
		REM Examine character & calculate offset from lookup tables
		REM This is brute force method & time consuming & a better method needs to be found
		REM ASCII values could be mathematically manipulated to come up with designed Feld Hell number
		select case jChar
		
			case " "
				Character_Index = 0
		
			case "A"
				Character_Index = 7
			
			case "B"
				Character_Index = 14
				
			case "C"
				Character_Index = 21
				
			case "D"
				Character_Index = 28
				
			case "E"
				Character_Index = 35
				
			case "F"
				Character_Index = 42
				
			case "G"
				Character_Index = 49
				
			case "H"
				Character_Index = 56
				
			case "I"
				Character_Index = 63
				
			case "J"
				Character_Index = 70
			
			case "K"
				Character_Index = 77
				
			case "L"
				Character_Index = 84
				
			case "M"
				Character_Index = 91
				
			case "N"
				Character_Index = 98
				
			case "O"
				Character_Index = 105
				
			case "P"
				Character_Index = 112
				
			case "Q"
				Character_Index = 119
				
			case "R"
				Character_Index = 126
				
			case "S"
				Character_Index = 133
				
			case "T"
				Character_Index = 140
				
			case "U"
				Character_Index = 147
				
			case "V"
				Character_Index = 154
				
			case "W"
				Character_Index = 161
				
			case "X"
				Character_Index = 168
				
			case "Y"
				Character_Index = 175
				
			case "Z"
				Character_Index = 182
				
			case "0"
				Character_Index = 189
				
			case "1"
				Character_Index = 196
				
			case "2"
				Character_Index = 203
				
			case "3"
				Character_Index = 210
				
			case "4"
				Character_Index = 217
				
			case "5"
				Character_Index = 224
				
			case "6"
				Character_Index = 231
				
			case "7"
				Character_Index = 238
				
			case "8"
				Character_Index = 245
				
			case "9"
				Character_Index = 252
				
			case "*"
				Character_Index = 259
					
			case "."
				Character_Index = 266
				
			case "?"
				Character_Index = 273
				
			case "!"
				Character_Index = 280
				
			case "("
				Character_Index = 287
				
			case ")"
				Character_Index = 294
				
			case "#"
				Character_Index = 301
				
			case "$"
				Character_Index = 308
				
			case "/"
				Character_Index = 315
			
			case "@"
				Character_Index = 322
							
		end select
		
		REM Check for valid character or not
		if Character_Index <> 9999 then
		
			gosub FeldHell				'it's valid so transmit it!
		
		end if
		
	next jLoop							'get next character in EEPROM to transmit
	
goto Beacon								'all done so restart transmission sequence & loop forever!
		

REM From here we get Hell character font data to build character transmission sequence
REM Because PICAXE only supports lookup tables of 256 bytes, we must use two for all characters in our font list
FeldHell:
	
	REM Is character in lookup table 1 (space & alphabet)?
	if Character_Index < 189 then
		Character_Offset = Character_Index + 6	'add in loop counter column offset
	
		REM Find column values in lookup table
		for kLoop = Character_Index to Character_Offset
			
			REM Lookup data table 1 for Hell character font space & alphabet characters (values in hexidecimal)
			lookup kLoop, (0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x07fc, 0x0e60, 0x0c60, 0x0e60, 0x07fc, 0x0000, 0x0000, _
					 0x0c0c, 0x0ffc, 0x0ccc, 0x0ccc, 0x0738, 0x0000, 0x0000, _
					 0x0ffc, 0x0c0c, 0x0c0c, 0x0c0c, 0x0c0c, 0x0000, 0x0000, _
					 0x0c0c, 0x0ffc, 0x0c0c, 0x0c0c, 0x07f8, 0x0000, 0x0000, _
					 0x0ffc, 0x0ccc, 0x0ccc, 0x0c0c, 0x0c0c, 0x0000, 0x0000, _
					 0x0ffc, 0x0cc0, 0x0cc0, 0x0c00, 0x0c00, 0x0000, 0x0000, _
					 0x0ffc, 0x0c0c, 0x0c0c, 0x0ccc, 0x0cfc, 0x0000, 0x0000, _
					 0x0ffc, 0x00c0, 0x00c0, 0x00c0, 0x0ffc, 0x0000, 0x0000, _
					 0x0000, 0x0c0c, 0x0ffc, 0x0c0c, 0x0000, 0x0000, 0x0000, _
					 0x003c, 0x000c, 0x000c, 0x000c, 0x0ffc, 0x0000, 0x0000, _
					 0x0ffc, 0x00c0, 0x00e0, 0x0330, 0x0e1c, 0x0000, 0x0000, _
					 0x0ffc, 0x000c, 0x000c, 0x000c, 0x000c, 0x0000, 0x0000, _
					 0x0ffc, 0x0600, 0x0300, 0x0600, 0x0ffc, 0x0000, 0x0000, _
					 0x0ffc, 0x0700, 0x01c0, 0x0070, 0x0ffc, 0x0000, 0x0000, _
					 0x0ffc, 0x0c0c, 0x0c0c, 0x0c0c, 0x0ffc, 0x0000, 0x0000, _
					 0x0c0c, 0x0ffc, 0x0ccc, 0x0cc0, 0x0780, 0x0000, 0x0000, _
					 0x0ffc, 0x0c0c, 0x0c3c, 0x0ffc, 0x000f, 0x0000, 0x0000, _
					 0x0ffc, 0x0cc0, 0x0cc0, 0x0cf0, 0x079c, 0x0000, 0x0000, _
					 0x078c, 0x0ccc, 0x0ccc, 0x0ccc, 0x0c78, 0x0000, 0x0000, _
					 0x0c00, 0x0c00, 0x0ffc, 0x0c00, 0x0c00, 0x0000, 0x0000, _
					 0x0ff8, 0x000c, 0x000c, 0x000c, 0x0ff8, 0x0000, 0x0000, _
					 0x0ffc, 0x0038, 0x00e0, 0x0380, 0x0e00, 0x0000, 0x0000, _
					 0x0ff8, 0x000c, 0x00f8, 0x000c, 0x0ff8, 0x0000, 0x0000, _
					 0x0e1c, 0x0330, 0x01e0, 0x0330, 0x0e1c, 0x0000, 0x0000, _
					 0x0e00, 0x0380, 0x00fc, 0x0380, 0x0e00, 0x0000, 0x0000, _
					 0x0c1c, 0x0c7c, 0x0ccc, 0x0f8c, 0x0e0c, 0x0000, 0x0000), Hell_Character
		
			gosub Transmit				'transmit each column (7 total)
		
		next kLoop						'loop through 7 columns
			 
	REM Character must be in lookup table 2 (numbers, punctuaton & special characters)
	else									
		
		Character_Index = Character_Index - 189	'shift values down since we can't exceed integer value 256
		Character_Offset = Character_Index + 6	'add in loop counter column offset
				 
		REM Find column values in ookup table 2
		for kLoop = Character_Index to Character_Offset
		
			REM Lookup data table for Hell character font (values in hexidecimal)
			Lookup kLoop, (0x07f8, 0x0c0c, 0x0c0c, 0x0c0c, 0x07f8, 0x0000, 0x0000, _
					 0x0300, 0x0600, 0x0ffc, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x061c, 0x0c3c, 0x0ccc, 0x078c, 0x000c, 0x0000, 0x0000, _
					 0x0006, 0x1806, 0x198c, 0x1f98, 0x00f0, 0x0000, 0x0000, _
					 0x1fe0, 0x0060, 0x0060, 0x0ffc, 0x0060, 0x0000, 0x0000, _
					 0x000c, 0x000c, 0x1f8c, 0x1998, 0x18f0, 0x0000, 0x0000, _
					 0x07fc, 0x0c66, 0x18c6, 0x00c6, 0x007c, 0x0000, 0x0000, _
					 0x181c, 0x1870, 0x19c0, 0x1f00, 0x1c00, 0x0000, 0x0000, _
					 0x0f3c, 0x19e6, 0x18c6, 0x19e6, 0x0f3c, 0x0000, 0x0000, _
					 0x0f80, 0x18c6, 0x18cc, 0x1818, 0x0ff0, 0x0000, 0x0000, _
					 0x018c, 0x0198, 0x0ff0, 0x0198, 0x018c, 0x0000, 0x0000, _
					 0x001c, 0x001c, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x1800, 0x1800, 0x19ce, 0x1f00, 0x0000, 0x0000, 0x0000, _
					 0x1f9c, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x01e0, 0x0738, 0x1c0e, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x1c0e, 0x0738, 0x01e0, 0x0000, 0x0000, 0x0000, 0x0000, _
					 0x0330, 0x0ffc, 0x0330, 0x0ffc, 0x0330, 0x0000, 0x0000, _
					 0x078c, 0x0ccc, 0x1ffe, 0x0ccc, 0x0c78, 0x0000, 0x0000, _
					 0x001c, 0x0070, 0x01c0, 0x0700, 0x1c00, 0x0000, 0x0000, _
					 0x07f8, 0x0c0c, 0x0ccc, 0x0d2c, 0x07c8, 0x0000, 0x0000), Hell_Character
 					
			gosub Transmit		'transmit each column (7 total)
		
		next kLoop				'loop through 7 columns

	end if
	
return						'exit & go back for another character to transmit
	
	
REM Routine to turn CW transmitter on & off by pulling PTT keying circuit rapidly
REM Defined Feld Hell character set right bit shifted (division by 2)
REM Other Feld Hell character sets may use left bit shifting (multiplication by 2)
Transmit:
	for Hell_Column_Pixel = 1 to 14	'loop through each column picture element (14 total)
 		
		REM For left bit shfting character fonts change Bit1 to Bit13
		If Bit1 = 1 then 			'Bit1 is 1st bit of word W0 or Hell_Character to which we will shift each of the column elements (pixels)
			
			high PTT			'turn on transmitter
		else
		
			low PTT  			'else turn off transmitter.
		
		end if
	
		REM Normal delay is 4.0815ms for Feld Hell pixel transmission
		REM Our delay is 3.063ms to compensate for processing & transmitter switch on/off delay (PIXIE II, in this case)
		REM Total character transmit time should be around 400ms (98 bits * 4.0815 ms)
		pauseus 2450			'for M2 chip @ 32MHz clock = 2450 * 1.25 = 3062.50us or 3.063ms
		
		REM For left bit shifting character fonts change to * 2
		Hell_Character = Hell_Character / 2 'shifts bits right to push next pixel value into Bit1 & drops off the previously used one
								 
	next Hell_Column_Pixel			'process next pixel in column
	
return						'back to calling routine
 

REM ************************************************************
REM PICAXE BASIC Feld Hell Character Set (values in hexidecimal)

REM Lookup table 1
REM ' ' 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
REM 'A' 0x07fc, 0x0e60, 0x0c60, 0x0e60, 0x07fc, 0x0000, 0x0000
REM 'B' 0x0c0c, 0x0ffc, 0x0ccc, 0x0ccc, 0x0738, 0x0000, 0x0000
REM 'C' 0x0ffc, 0x0c0c, 0x0c0c, 0x0c0c, 0x0c0c, 0x0000, 0x0000
REM 'D' 0x0c0c, 0x0ffc, 0x0c0c, 0x0c0c, 0x07f8, 0x0000, 0x0000
REM 'E' 0x0ffc, 0x0ccc, 0x0ccc, 0x0c0c, 0x0c0c, 0x0000, 0x0000
REM 'F' 0x0ffc, 0x0cc0, 0x0cc0, 0x0c00, 0x0c00, 0x0000, 0x0000
REM 'G' 0x0ffc, 0x0c0c, 0x0c0c, 0x0ccc, 0x0cfc, 0x0000, 0x0000
REM 'H' 0x0ffc, 0x00c0, 0x00c0, 0x00c0, 0x0ffc, 0x0000, 0x0000
REM 'I' 0x0000, 0x0c0c, 0x0ffc, 0x0c0c, 0x0000, 0x0000, 0x0000
REM 'J' 0x003c, 0x000c, 0x000c, 0x000c, 0x0ffc, 0x0000, 0x0000
REM 'K' 0x0ffc, 0x00c0, 0x00e0, 0x0330, 0x0e1c, 0x0000, 0x0000
REM 'L' 0x0ffc, 0x000c, 0x000c, 0x000c, 0x000c, 0x0000, 0x0000
REM 'M' 0x0ffc, 0x0600, 0x0300, 0x0600, 0x0ffc, 0x0000, 0x0000
REM 'N' 0x0ffc, 0x0700, 0x01c0, 0x0070, 0x0ffc, 0x0000, 0x0000
REM 'O' 0x0ffc, 0x0c0c, 0x0c0c, 0x0c0c, 0x0ffc, 0x0000, 0x0000
REM 'P' 0x0c0c, 0x0ffc, 0x0ccc, 0x0cc0, 0x0780, 0x0000, 0x0000
REM 'Q' 0x0ffc, 0x0c0c, 0x0c3c, 0x0ffc, 0x000f, 0x0000, 0x0000
REM 'R' 0x0ffc, 0x0cc0, 0x0cc0, 0x0cf0, 0x079c, 0x0000, 0x0000
REM 'S' 0x078c, 0x0ccc, 0x0ccc, 0x0ccc, 0x0c78, 0x0000, 0x0000
REM 'T' 0x0c00, 0x0c00, 0x0ffc, 0x0c00, 0x0c00, 0x0000, 0x0000
REM 'U' 0x0ff8, 0x000c, 0x000c, 0x000c, 0x0ff8, 0x0000, 0x0000
REM 'V' 0x0ffc, 0x0038, 0x00e0, 0x0380, 0x0e00, 0x0000, 0x0000
REM 'W' 0x0ff8, 0x000c, 0x00f8, 0x000c, 0x0ff8, 0x0000, 0x0000
REM 'X' 0x0e1c, 0x0330, 0x01e0, 0x0330, 0x0e1c, 0x0000, 0x0000
REM 'Y' 0x0e00, 0x0380, 0x00fc, 0x0380, 0x0e00, 0x0000, 0x0000
REM 'Z' 0x0c1c, 0x0c7c, 0x0ccc, 0x0f8c, 0x0e0c, 0x0000, 0x0000

REM Lookup table 2
REM '0' 0x07f8, 0x0c0c, 0x0c0c, 0x0c0c, 0x07f8, 0x0000, 0x0000
REM '1' 0x0300, 0x0600, 0x0ffc, 0x0000, 0x0000, 0x0000, 0x0000
REM '2' 0x061c, 0x0c3c, 0x0ccc, 0x078c, 0x000c, 0x0000, 0x0000
REM '3' 0x0006, 0x1806, 0x198c, 0x1f98, 0x00f0, 0x0000, 0x0000
REM '4' 0x1fe0, 0x0060, 0x0060, 0x0ffc, 0x0060, 0x0000, 0x0000
REM '5' 0x000c, 0x000c, 0x1f8c, 0x1998, 0x18f0, 0x0000, 0x0000
REM '6' 0x07fc, 0x0c66, 0x18c6, 0x00c6, 0x007c, 0x0000, 0x0000
REM '7' 0x181c, 0x1870, 0x19c0, 0x1f00, 0x1c00, 0x0000, 0x0000
REM '8' 0x0f3c, 0x19e6, 0x18c6, 0x19e6, 0x0f3c, 0x0000, 0x0000
REM '9' 0x0f80, 0x18c6, 0x18cc, 0x1818, 0x0ff0, 0x0000, 0x0000
REM '*' 0x018c, 0x0198, 0x0ff0, 0x0198, 0x018c, 0x0000, 0x0000
REM '.' 0x001c, 0x001c, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
REM '?' 0x1800, 0x1800, 0x19ce, 0x1f00, 0x0000, 0x0000, 0x0000
REM '!' 0x1f9c, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
REM '(' 0x01e0, 0x0738, 0x1c0e, 0x0000, 0x0000, 0x0000, 0x0000
REM ')' 0x1c0e, 0x0738, 0x01e0, 0x0000, 0x0000, 0x0000, 0x0000
REM '#' 0x0330, 0x0ffc, 0x0330, 0x0ffc, 0x0330, 0x0000, 0x0000
REM '$' 0x078c, 0x0ccc, 0x1ffe, 0x0ccc, 0x0c78, 0x0000, 0x0000
REM '/' 0x001c, 0x0070, 0x01c0, 0x0700, 0x1c00, 0x0000, 0x0000
REM '@' 0x07f8, 0x0c0c, 0x0ccc, 0x0d2c, 0x07c8, 0x0000, 0x0000