REM **********************************************************
REM * PICAXE Morse Code CW Beacon Program v1.0, October 2012 *
REM * Based on original PICAXE Program by Steve, VK6HV, 2008 *
REM **********************************************************

REM Various modifications by Robert, VA3ROM, http://my.tbaytel.net/va3rom

REM For use with connecting CW transceivers to Rev. Ed. Schools Experimenter PICAXE 08M/08M2 Kit
REM Will work with both PICAXE 08M/08M2 MCUs & others (by converting program)
REM by porting code over for that MCU platform (Arduino, BASIC Stamp) & wiring applicable ports

REM ***********************************************************

REM Define program Morse code speed & timing units (2 bytes each) adjust with pot connected to ADC
symbol WPM = b5							'transmission speed variable

symbol DIT_Time = w4						'dit length timing variable
symbol DAH_Time = w5						'dah length timing variable
symbol WORD_Time = w6						'word spacing timing variable

REM Define program variables (The LED and PTT variables are defined for used with Schools Kit
REM Change for your MCU & circuit requirements, as required if not uscing Schools Kit
REM Set all Schools Experimenter Kit DIP switches to "OFF" (down) positions (usually)!
REM To hear transmitted beacon audio set DIP switch 2 to "ON" postiion
REM The PIXACE 08M2 sound is very square wave & raspy sounding compared to more sine wave sounding 08M chip
REM A simple RC filter place in the Piezo speaker circuit could fix that
symbol PTT = C.0	 						'PTT transmitter keying circuit on port 0 (red LED on Schools Kit) 
									'red LED will flash in sync with Morse code & activate off board optocoupler circuit

symbol PIEZO = C.2						'Piezo speaker on port 2 (shared with green LED on Schools Kit)

symbol Character = b0						'Morse code character
symbol Even_Or_Odd_Character = b1				'even or odd flag (dit or dah)
symbol Index = b2							'index pointer to eeprom memory character to transmit
symbol Beacon_Length = b3					'number of characters in beacon - 1
symbol CW_Tone = b4						'Morse code audio tone for PIEZO to use

REM ***********************************************************
REM Create your Morse code beacon as required (max 256 bytes for 08M2 & 80 for 08M
REM See encode/decode table at end of this program

REM This is a sample "HELLO, WORLD!" beacon & yes, there is an exclamation Morse code symbol!
eeprom 0,(82,16,2,18,18,15,115,0,14,15,10,18,9,117,82)

Beacon_Length = 14						'number of beacon characters - 1 (minus 1, very important!)

WPM = 20								'set default Morse code transmission speed fixed if not using ADC speed control potentiometer

CW_Tone = 112							'set default tone for Piezo speaker (0 to 128)
REM ************************************************************


REM Start program Morse code beacon transmission sequence
Main: 
	low PTT							'turn off PTT & LEDs (set to known state)
			
	wait 2							'2-second silence at start/end of beacon sequence
	  
REM VSM voltage divider 10K pot sits on ADC port 4 & used as speed control
REM Uncomment this block of code if using fixed transmission speed & not using VSM's ADC potentiometer
	'readADC 4, WPM                                 	'read potentiometer ADC 8-bit value (0 to 255) to get millisecond dit time
     
   	'if WPM = 0 then						'prevent divisioin by 0!
   	'	WPM = 1 
   	'end if 
  		
      'WPM = 1000 / WPM						'convert ADC millisecond dit time value to words per minute
									'25ms = 40WPM 50ms =s 20WPM 100ms = 10WPM 200 = 5WPM (WPM = words per minute)        	
        	
REM Calculate Morse code routine variables based on WPM (ADC or manual setting) to usee
REM Time (length) for base unit (dit) & also intra-character character element spacing beteen dits & dahs
REM Timing values based on Morse code "PARIS" calibration standard
	DIT_Time = 100 / WPM					'conversion to sound statements' durations in 10ms increments!
									'rates are +/- 1 WPM

	DAH_Time = 3 * DIT_Time					'time (length) for 1 dah (dash)
	WORD_Time = 7 * DIT_Time				'time (space) between each word

	high PTT							'turn on CW transmitter

	sound PIEZO, (CW_Tone, 250, CW_Tone, 250)		'sound Morse code 5 second (5000 milliseconds) zero beat tuning tone
	
	low PTT							'turn off transmitter
	
	wait 2							'2-second silence before sending beacon ID, grid & other info

	for Index = 0 to Beacon_Length                 	'start transmission of my beacon string

		read Index, Character                     'read EEPROM memory & get next transmit character 

		gosub Morse						'send it
	
	next Index							'repeat until all characters transmitted
	
goto Main 		                         		'go back to start of beacon sequence & repeat forever


REM **********************************************************************
REM * Routine to transmit Morse code characters using right bit shifting *
REM * Based on orignal code by Steve Paige, VK6HV.                       *
REM **********************************************************************
Morse:  							
	REM Check for word space character (not used with this program kept in for proper timing of Morse routne)
	if Character = 0 then
		
		gosub Word_Space
		
		return
	
	end if		
	
	REM Right bit shifting (division) routine will keep going until Morse code character value reaches 1
	do

		Even_Or_Odd_Character = Character // 2	'is character value even or odd (returns remainder 1 or 0, odd or even (dah or dit))

  		Character = Character / 2                 'division by 2 performs binary right bit shift
  			  			  
	     	if Even_Or_Odd_Character = 1 then		'if result is odd 1 (dah) else even 0 (dit)

		 	gosub Dah			'send dah (dash) character element
 	      
	     	else
	     		
			gosub Dit			'send dit (dot) character element
	
  		end if
  	
  		sound PIEZO, (0, DIT_Time)	'wait 1 dit length for proper intra character spacing
			
  	loop until Character = 1		'keep looping until all bits shifted & value of 1 is reached
	  		  
  	sound PIEZO, (0, DAH_Time)		'wait 3 dit lengths for proper inter character spacing

return						'back to main program & get next character to send

REM Dit routine to sound a dit (dot)
Dit:
	high PTT					'turn on transmitter

	sound PIEZO, (CW_Tone, DIT_Time)	'sound Morse code dit
	
	'pause DIT_Time				'keep it on for one dit time unit

	low PTT					'turn off transmitter
		
	return					'back to point of call

REM Dah routine to sound a dah (dash)
Dah:
	high PTT					'turn on transmitter

	sound PIEZO, (CW_Tone, DAH_Time)	'sound Morse code dah
	
	'pause DAH_Time				'keep it on for one dah time unit 

	low PTT					'turn of transmitter
	
	return					'back to point of call

REM Word space routine to pause between words
Word_Space:
	sound PIEZO, (0, WORD_Time)
	
	return					'back to point of call


REM **************************** MORSE CODE CHARACTERS ***********************************

REM      A .-    B -...  C -.-.  D -..    E .      F ..-.
REM      G --.   H ....  I ..    J .---   K -.-    L .-..
REM      M --    N -.    O ---   P .--.   Q --.-   R .-.
REM      S ...   T -     U ..-   V ...-   W .--    X -..-
REM      Y -.--  Z --.. 

REM      1 .---- 2 ..--- 3 ...-- 4 ....-  5 .....
REM      6 -.... 7 --... 8 ---.. 9 ----.  0 -----

REM      FULL STOP          [.]  .-.-.-     COLON                   [:]  ---...
REM      COMMA              [,]  --..--     SEMICOLON               [;]  -.-.-.
REM      QUESTION MARK      [?]  ..--..     EQUAL SIGN (DOUBLE DASH)[=]  -...-
REM      APOSTROPHE         [']  .----.     PLUS                    [+]  .-.-.
REM      EXCLAMATION MARK   [!]  -.-.--     HYPHEN minus            [-]  -....-
REM      FWD SLASH          [/]  -..-.      UNDERSCORE              [_]  ..--.-
REM      PARENTHESIS open   [(]  -.--.      QUOTATION MARK          ["]  .-..-.
REM      PARENTHESIS closed [)]  -.--.-     DOLLAR SIGN             [$]  ...-..-
REM      AMPERSAND          [&]  .-...      AT SIGN (COMMAT) [WR]   [@]  .--.-.

REM ******************* ENCODED MORSE CODE ALPHANUMERIC CHARACTER SET ********************

REM      A   6,   B  17,   C  21,   D   9,   E   2,    F  20,
REM      G  11,   H  16,   I   4,   J  30,   K  13,    L  18,
REM      M   7,   N   5,   O  15,   P  22,   Q  27,    R  10,
REM      S   8,   T   3,   U  12,   V  24,   W  14,    X  25,
REM      Y  29,   Z  19, 
	   
REM      1   62,   2  60,   3  56,   4  48,   5   32,  
REM      6   33,   7  35,   8  39,   9  47,   0   63,

REM      FULL STOP          [.]  106     COLON                  [:]  71
REM      COMMA              [,]  115     SEMICOLON              [;]  85
REM      QUESTION MARK      [?]  76      EQUAL SIGN(DOUBLE DASH)[=]  49
REM      APOSTROPHE         [']  94      PLUS                   [+]  42
REM      EXCLAMATION MARK   [!]  117     HYPHEN minus           [-]  97
REM      FWD SLASH          [/]  41      UNDERSCORE             [_]  108
REM      PARENTHESIS open   [(]  45      QUOTATION MARK         ["]  82
REM      PARENTHESIS closed [)]  109     DOLLAR SIGN            [$]  200
REM      AMPERSAND          [&]  34      AT SIGN (COMMAT) [WR]  [@]  86		SPACE [ ] 0

REM **************************************************************************************
